import { jsx as _jsx } from "react/jsx-runtime";
/**
 *
 * :::warning Note
 * This is the documentation for `next-auth@latest`. Check out the documentation of v4 [here](https://next-auth.js.org).
 * :::
 *
 * If you are looking for the migration guide, visit the [`next-auth@latest` Migration Guide](https://nextjs.authjs.dev/v5).
 *
 * ## Installation
 *
 * ```bash npm2yarn2pnpm
 * npm install next-auth@5 @auth/core
 * ```
 *
 * ## Signing in and signing out
 *
 * The App Router embraces Server Actions that can be leveraged to decrease the amount of JavaScript sent to the browser.
 *
 * :::info
 * [Next.js Server Actions](https://nextjs.org/docs/app/building-your-application/data-fetching/server-actions) is **in alpha stage**. In the future, NextAuth.js will integrate with Server Actions and provide first-party APIs.
 * Until then, here is how you can use NextAuth.js to log in and log out without JavaScript.
 * :::
 *
 * ```ts title="app/auth-components.tsx"
 * import { CSRF_experimental } from "../auth"
 *
 * export function SignIn({ provider, ...props }: any) {
 *   return (
 *     <form action={`/api/auth/signin/${provider}`} method="post">
 *       <button {{...props}}/>
 *       <CSRF_experimental/>
 *     </form>
 *   )
 * }
 *
 * export function SignOut(props: any) {
 *   return (
 *     <form action="/api/auth/signout" method="post">
 *       <button {...props}/>
 *       <CSRF_experimental/>
 *     </form>
 *   )
 * }
 * ```
 *
 * Alternatively, you can create client components, using the `signIn()` and `signOut` methods:
 *
 * ```ts title="app/auth-components.tsx"
 * "use client"
 * import { signIn, signOut } from "next-auth/react"
 *
 * export function SignIn({provider, ...props}: any) {
 *   return <button {...props} onClick={() => signIn(provider)}/>
 * }
 *
 * export function SignOut(props: any) {
 *   return <button {...props} onClick={() => signOut()}/>
 * }
 * ```
 *
 * Then, you could for example use it like this:
 *
 * ```ts title=app/page.tsx
 * import { SignIn, SignOut } from "./auth-components"
 *
 * export default async function Page() {
 *   const session = await auth()
 *   if (session) {
 *     return (
 *       <>
 *         <pre>{JSON.stringify(session, null, 2)}</pre>
 *         <SignOut>Sign out</SignOut>
 *       </>
 *     )
 *   }
 *   return <SignIn provider="github">Sign in with GitHub</SignIn>
 * }
 * ```
 *
 *
 * ## Environment variable inferrence
 *
 * `NEXTAUTH_URL` and `NEXTAUTH_SECRET` have been inferred since v4.
 *
 * Since NextAuth.js v5 can also automatically infer environment variables that are prefiexed with `AUTH_`.
 *
 * For example `AUTH_GITHUB_ID` and `AUTH_GITHUB_SECRET` will be used as the `clientId` and `clientSecret` options for the GitHub provider.
 *
 * :::tip
 * The environment variable name inferring has the following format for OAuth providers: `AUTH_{PROVIDER}_{ID|SECRET}`.
 *
 * `PROVIDER` is the uppercase snake case version of the provider's id, followed by either `ID` or `SECRET` respectively.
 * :::
 *
 * `AUTH_SECRET` and `AUTH_URL` are also aliased for `NEXTAUTH_SECRET` and `NEXTAUTH_URL` for consistency.
 *
 * To add social login to your app, the configuration becomes:
 *
 * ```ts title="auth.ts"
 * import NextAuth from "next-auth"
 * import GitHub from "next-auth/providers/GitHub"
 * export const { handlers, auth } = NextAuth({ providers: [ GitHub ] })
 * ```
 *
 * And the `.env.local` file:
 *
 * ```sh title=".env.local"
 * AUTH_GITHUB_ID=...
 * AUTH_GITHUB_SECRET=...
 * AUTH_SECRET=...
 * ```
 *
 * :::tip
 * In production, `AUTH_SECRET` is a required environment variable - if not set, NextAuth.js will throw an error. See [MissingSecretError](https://authjs.dev/reference/core/errors#missingsecret) for more details.
 * :::
 *
 * If you need to override the default values for a provider, you can still call it as a function `GitHub({...})` as before.
 *
 * @module index
 */
import { Auth, skipCSRFCheck, raw } from "@auth/core";
import { cookies, headers } from "next/headers";
import { detectOrigin, reqWithEnvUrl, setEnvDefaults } from "./lib/env.js";
import { initAuth } from "./lib/index.js";
import { redirect } from "next/navigation";
/**
 *  Initialize NextAuth.js.
 *
 *  @example
 * ```ts title="auth.ts"
 * import NextAuth from "next-auth"
 * import GitHub from "@auth/core/providers/github"
 *
 * export const { handlers, auth } = NextAuth({ providers: [GitHub] })
 * ```
 */
export default function NextAuth(config) {
    setEnvDefaults(config);
    const httpHandler = (req) => Auth(reqWithEnvUrl(req), config);
    return {
        handlers: { GET: httpHandler, POST: httpHandler },
        // @ts-expect-error
        auth: initAuth(config),
        async CSRF_experimental() {
            const _headers = headers();
            const url = `${detectOrigin(_headers).origin}/api/auth/csrf`;
            const value = await Auth(new Request(url, { headers: _headers }), config)
                .then((res) => res.json())
                .then((res) => res?.csrfToken);
            if (!value)
                throw new Error("CSRF token could not be found.");
            return _jsx("input", { type: "hidden", name: "csrfToken", value: value });
        },
        signIn(provider, params) {
            return async () => {
                "use server";
                const _headers = new Headers(headers());
                _headers.set("Content-Type", "application/x-www-form-urlencoded");
                const origin = detectOrigin(_headers);
                const url = `${origin}/signin/${provider}`;
                const res = (await Auth(new Request(url, {
                    method: "POST",
                    headers: _headers,
                    body: new URLSearchParams({
                        callbackUrl: params?.redirectTo ?? _headers.get("Referer") ?? "/",
                    }),
                }), { ...config, raw, skipCSRFCheck })); // TODO: fix return type when "raw" is set
                for (const c of res?.cookies ?? []) {
                    cookies().set(c.name, c.value, c.options);
                }
                if (params?.redirect)
                    return redirect(res.redirect);
                return res.redirect;
            };
        },
        signOut(params) {
            return async () => {
                "use server";
                const _headers = new Headers(headers());
                _headers.set("Content-Type", "application/x-www-form-urlencoded");
                const origin = detectOrigin(_headers);
                const url = `${origin}/signout`;
                const res = (await Auth(new Request(url, {
                    method: "POST",
                    headers: _headers,
                    body: new URLSearchParams({
                        callbackUrl: params?.redirectTo ?? _headers.get("Referer") ?? "/",
                    }),
                }), { ...config, raw, skipCSRFCheck })); // TODO: fix return type when "raw" is set
                for (const c of res?.cookies ?? []) {
                    cookies().set(c.name, c.value, c.options);
                }
                if (params?.redirect)
                    return redirect(res.redirect);
                return res.redirect;
            };
        },
    };
}
